"use strict";
/*----------------------------------
- DEPENDENCIES
----------------------------------*/
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
exports.isValueExtractor = void 0;
// Npm
var get_root_domain_1 = __importDefault(require("get-root-domain"));
var isURL_1 = __importDefault(require("validator/lib/isURL"));
// Interval
var types_1 = require("./types");
var BadRequest = /** @class */ (function (_super) {
    __extends(BadRequest, _super);
    function BadRequest() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return BadRequest;
}(Error));
/*----------------------------------
- CONFIG
----------------------------------*/
var reqPerCall = 3;
/*----------------------------------
- METHODS
----------------------------------*/
exports["default"] = (function (requests) {
    // Type Check
    if (!Array.isArray(requests))
        throw new BadRequest("requests must be an array. Provided: " + typeof requests);
    // Requests number / call
    var reqCount = requests.length;
    if (reqCount === 0)
        throw new BadRequest("You must provide at least one request.");
    if (reqCount > reqPerCall)
        throw new BadRequest("You can't send more than " + reqPerCall + " requests per call (" + reqCount + " given).");
    // Check every request
    var domains = {};
    for (var iReq = 0; iReq < reqCount; iReq++) {
        var req = requests[iReq];
        // Type
        if (typeof req !== "object" || req === null)
            throw new BadRequest("requests must be an array of requests object, but requests[" + iReq + "] is an " + typeof req);
        // Method
        if (req.method === undefined)
            req.method = 'GET';
        else if (!types_1.allowedMethods.includes(req.method))
            throw new BadRequest("Only the following HTTP methods are currently allowed: " + types_1.allowedMethods.join(', '));
        // URL
        if (typeof req.url !== "string" || !(0, isURL_1["default"])(req.url, {
            require_protocol: true,
            require_valid_protocol: true,
            protocols: ['http', 'https'],
            require_host: true,
            require_port: false,
            allow_protocol_relative_urls: false
        }))
            throw new BadRequest("The url parameter must be a valid URL string: protocol (http or https) + domain + path (optional)");
        // Unique domain
        if (reqCount !== 1) {
            var domain = (0, get_root_domain_1["default"])(req.url);
            if (domains[domain] === true)
                throw new BadRequest("When you send multiple requests in one call, each requets must point to different domain names. However, you're sending 2 requests to " + domain + ".");
            domains[domain] = true;
        }
        // Cookies
        if (req.cookies !== undefined) {
            // Type
            if (typeof req.cookies !== 'string')
                throw new BadRequest("The cookie parameter must be a string. Example: user=Bob; age=28;");
        }
        // body
        if (req.body !== undefined) {
            // Bodytype
            if (req.bodyType === undefined)
                throw new BadRequest("The bodyType parameter must be provided when the body parameter is specified.");
            if (!types_1.bodyTypes.includes(req.bodyType))
                throw new BadRequest("Invalid value for the bodyType parameter. Allowed values: " + types_1.bodyTypes.join(', '));
            // Type
            if (typeof req.body !== 'object' || req.body.constructor.name !== 'Object')
                throw new BadRequest("The body parameter must be an object.");
        }
        if (req.extract !== undefined)
            req.extract = validateExtractors(req.extract, 'extract');
        if (req.withBody !== undefined && typeof req.withBody !== "boolean")
            throw new BadRequest("The withBody parameter must be a boolean.");
        if (req.withHeaders !== undefined && typeof req.withHeaders !== "boolean")
            throw new BadRequest("The withHeaders parameter must be a boolean.");
    }
    return requests;
});
var isValueExtractor = function (extract) {
    return ('select' in extract);
};
exports.isValueExtractor = isValueExtractor;
var validateExtractors = function (extract, path) {
    var e_1, _a;
    if (!extract || Array.isArray(extract) || typeof extract !== 'object')
        throw new BadRequest("The " + path + " option must be an object (" + typeof extract + " given).");
    if (extract instanceof types_1.ValueExtractor)
        extract = extract.options;
    // The two required options in TValueExtractor
    if ((0, exports.isValueExtractor)(extract)) {
        /*
            extract: {
                select: "h4",
                attr: "text",
                required: true,
                filters: ["title"]
            }
        */
        var select = extract.select, _b = extract.attr, attr = _b === void 0 ? 'text' : _b, _c = extract.required, required = _c === void 0 ? true : _c, _d = extract.filters, filters = _d === void 0 ? [] : _d;
        if (typeof select !== 'string')
            throw new BadRequest("The ".concat(path, ".select option must be a string."));
        if (typeof attr !== 'string')
            throw new BadRequest("The ".concat(path, ".attr option must be a string."));
        if (typeof required !== 'boolean')
            throw new BadRequest("The ".concat(path, ".required option must be a boolean."));
        if (!Array.isArray(filters))
            throw new BadRequest("The ".concat(path, ".filters option must be an array of strings."));
        try {
            for (var filters_1 = __values(filters), filters_1_1 = filters_1.next(); !filters_1_1.done; filters_1_1 = filters_1.next()) {
                var filter = filters_1_1.value;
                if (!types_1.dataFilters.includes(filter))
                    throw new BadRequest("The filter \"" + filter + "\" you gave in " + path + " does not exists. Possible filters: " + types_1.dataFilters.join(', '));
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (filters_1_1 && !filters_1_1.done && (_a = filters_1["return"])) _a.call(filters_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
    }
    else {
        /*
            extract: {
                "$foreach": ".sh-dgr__content",
                "name": {
                    select: "h4",
                    attr: "text",
                    required: true,
                    filters: ["title"]
                }
                ...
            }
        */
        if (Object.keys(extract).length === 0)
            throw new BadRequest("The " + path + " parameter must contain at least one entry.");
        // For each extractor
        for (var name_1 in extract) {
            // Foreach
            if (name_1 !== '$foreach')
                extract[name_1] = validateExtractors(extract[name_1], path + '.' + name_1);
            else if (typeof extract[name_1] !== "string")
                throw new BadRequest("When specified, the " + path + ".$foreach parameter must be a CSS selector string.");
        }
    }
    return extract;
};
